@@ -10,9 +10,10 @@
 #   where expr(g)=1 if g expressed value>T1, else 0
 #          flux(g)=0 if (all reactions catalyzed by g) have no flux(Threshold T2 and sum of all), 0 else.
 ############
-
+#Multifunctional enzymes is making a problem: no solution exists  4/2/2015
+# for multifunctional enzymes add a constraint g=g1 or g2 or g3 ....
 eFBA_gene <- function (model, expressionData,
-                  Tf =1e-5,#SYBIL_SETTINGS("TOLERANCE"),  # threshold on flux
+                  Tf =0.01,#SYBIL_SETTINGS("TOLERANCE"),  # threshold on flux
 				  pct_objective=100,
  		           lpdir = SYBIL_SETTINGS("OPT_DIRECTION"),
 			   solver = SYBIL_SETTINGS("SOLVER"),
@@ -80,7 +81,8 @@ return(eqn)
     
     #2- add new constraint for FB
     #3- set new obj function if expr(gi)=0 then add yi to obj else add -yi. 
-    #4- Add new n constraints(identifying yi's):  -Bi*yi <= vi <= Bi*yi
+    #4- Add new n constraints(identifying yi's): 2 for irreversible and 4 for reversible rxns
+	#5- Add constraints for multifunctional enzymes
 
         nc     <- react_num(model)
         nr     <- met_num(model)
@@ -125,13 +127,11 @@ return(eqn)
 		\n rxns that are always ON: %d ",sum(rxnfva),sum(abs(fv_min-fv_max)<Tf),sum(((fv_min>Tf) | (fv_max < -Tf)) & abs(fv_min-fv_max)>=Tf) ))
     	print("state after FVA:");
 		print(table(rxnStatus))
-		gpr_rxn_ind[rxnStatus==0]=F;
+		gpr_rxn_ind[rxnStatus==0]=FALSE;
 		rxnstname=ifelse(rxnStatus==0,"No rule/Unk",ifelse(rxnStatus==-1,"ON","OFF"));
 		print(table(rxnstname))
 		
-	    gpr_rxn=sum(gpr_rxn_ind);
-		rules=gpr(model)[gpr_rxn_ind]
-         
+	     
         #  the problem: minimize:
         #  
         #            |      |     
@@ -142,27 +142,48 @@ return(eqn)
         #        c^T |  0   | = FB
         #            |      |
         #       ------------------
-        #            |      |
-        #         -1 | -ub  | <= 0    ] 
-        #            |      |           } -ub*yi <= vi <= ub * yi
-        #         +1 | -ub  | <= 0    ]
-        #            |      |
+        #          identifying constraints of fi
+		#       ------------------
+		#       Linear constraints of GPR
         #       ------------------
         #  lb   wt_lb|  0   |
         #  ub   wt_ub|  1   |
         #            |      |
         #  obj    0  |  2*ei-1    where ei =0 gene i NOT expressed/ 1 otherwise
 
+        nGpr=sum(gpr_rxn_ind);
+		rules=gpr(model)[gpr_rxn_ind]
+        is_irrev=(gpr_rxn_ind & lowbnd(model)>=0);
+	    nIrrev=sum(is_irrev);
+		
+		is_rev=(gpr_rxn_ind & lowbnd(model)<0);
+        nRev=sum(is_rev);
+		INF=max(uppbnd(model))+1;
+		#--------------------identify variable for genes:05/02/2015--------------------------
+		colid=nc+nGpr+nRev+1;
+		geneCol=NULL;#column in problemcoresponding to given gene
 
-        # ---------------------------------------------
+		#Add variables for all genes
+		for( g in allGenes(model)){
+				cnt=length(grep(g,gpr(model)[gpr_rxn_ind]))#include only rules of reactions
+				geneCol=rbind(geneCol,cbind(gene=g,rxn=NA,Col=colid,cnt,varname=paste("g",g,sep="_"),inputStat=x[grep(g,allGenes(model))]))
+				colid=colid+1;
+				if(cnt>1){
+					for(r in grep(g,gpr(model)[gpr_rxn_ind]) ){
+						geneCol=rbind(geneCol,cbind(gene=g,rxn=(react_id(model)[gpr_rxn_ind])[r],Col=colid,cnt,varname=paste("g",g,r,sep="_"),inputStat=x[grep(g,allGenes(model))]))
+						colid=colid+1;			
+					}
+				}
+		}
+		nvg=colid-(nc+nGpr+nRev+1);
+		# ---------------------------------------------
         # constraint matrix
         # ---------------------------------------------
 
        # the initial matrix dimensions  variables:   nc:rates, nc:flux(r)(0,1) , ng
-       #sum((gpr(model)!="") & !react_rev(model)) + sum((gpr(model)!="") & react_rev(model)) * 2;
-      
-       #LHS <- as.matrix.csr(0, nrow = nr+gpr_rxn*3+1, ncol = (nc+gpr_rxn+ng))
-       LHS <- Matrix::Matrix(0, nrow = nr+gpr_rxn*3+1, ncol = (nc+gpr_rxn+ng), sparse = TRUE)
+	   # integer variables:fi(nGpr),yi(nRev),gene state ng
+	   # rows nIrrev:2, nRev:4  ,Gpr:nGpr         ---02/02/2015--------
+       LHS <- Matrix::Matrix(0, nrow = nr+nIrrev*2+nRev*4+nGpr+1, ncol = (nc+nGpr+nRev+nvg), sparse = TRUE)
 
        # rows for the initial S
        LHS[1:nr,1:nc] <- S(model)
@@ -172,57 +193,120 @@ return(eqn)
 
        # rows for the identifying constraint of flux variables
        #diag is wrong : it cant be seq, only the first 810 will be used regardless of gpr existence
-	if(gpr_rxn>0){
-
-       ii=matrix(c((nr+2)   :(nr+gpr_rxn+1)  ,((1:nc)[gpr_rxn_ind] )),ncol=2)
-       LHS[ii ]<-1
+	# Irreversible reactions 2 constraints
+	if(nIrrev>0){
+    #  Constraint 1: vi-M*fi<=Tf
+       ii=matrix(c((nr+2)   :(nr+nIrrev+1)  ,((1:nc)[is_irrev] )),ncol=2)
+       LHS[ii]<- 1
 		
-		if(gpr_rxn>1){
-			diag(LHS[(nr+2)   :(nr+gpr_rxn+1)  ,((1:nc)[gpr_rxn_ind] )   ]) <- -1
-			diag(LHS[(nr+gpr_rxn+2)   :(nr+2*gpr_rxn+1)  ,((1:nc)[gpr_rxn_ind] )    ]) <- 1
-
-			diag(LHS[(nr+2)   :(nr+gpr_rxn+1)  ,(nc+1)  :(nc+1+gpr_rxn)]) <- -uppbnd(model)[gpr_rxn_ind]
-			diag(LHS[(nr+gpr_rxn+2)   :(nr+2*gpr_rxn+1)  ,(nc+1)  :(nc+1+gpr_rxn) ]) <- -uppbnd(model)[gpr_rxn_ind]
+		if(nIrrev>1){
+			diag(LHS[(nr+2)   :(nr+nIrrev+1)  ,((nc+1):(nc+nIrrev))   ]) <- -INF   # M*fi
 		}else{# diag function fails when it is one row
-				LHS[(nr+2)   :(nr+gpr_rxn+1)  ,((1:nc)[gpr_rxn_ind] ) ] = -1
-				LHS[(nr+gpr_rxn+2)   :(nr+2*gpr_rxn+1)  ,((1:nc)[gpr_rxn_ind] )    ] <- 1
-                LHS[(nr+2)   :(nr+gpr_rxn+1)  ,(nc+1)  :(nc+1+gpr_rxn)] <- -uppbnd(model)[gpr_rxn_ind]
-			    LHS[(nr+gpr_rxn+2)   :(nr+2*gpr_rxn+1)  ,(nc+1)  :(nc+1+gpr_rxn) ] <- -uppbnd(model)[gpr_rxn_ind]
+			LHS[(nr+2)   :(nr+nIrrev+1)  ,((nc+1):(nc+nIrrev))  ] <- -INF
+		}
+		
+		#  Constraint 2: Tf*fi-vi<=0
+       ii=matrix(c((nr+nIrrev+2)   :(nr+2*nIrrev+1)  ,((1:nc)[is_irrev] )),ncol=2)
+       LHS[ii]<- -1
+		
+		if(nIrrev>1){
+			diag(LHS[(nr+nIrrev+2)   :(nr+2*nIrrev+1)  ,((nc+1):(nc+nIrrev))   ]) <- Tf   # Tf*fi
+		}else{# diag function fails when it is one row
+			LHS[(nr+nIrrev+2)   :(nr+2*nIrrev+1)  ,((nc+1):(nc+nIrrev))   ] <- Tf
+		}
+	}#Irrev
+	# Irreversible reactions 4 constraints, one additional integer variable
+	if(nRev>0){
+    #  Constraint 1: vi-M*fi<=Tf
+       ii=matrix(c((nr+2*nIrrev+2)   :(nr+2*nIrrev+nRev+1)  ,((1:nc)[is_rev] )),ncol=2)
+       LHS[ii]<- 1
+	    
+		if(nRev>1){
+			diag(LHS[(nr+2*nIrrev+2)   :(nr+2*nIrrev+nRev+1)  ,((nc+nIrrev+1):(nc+nIrrev+nRev))   ]) <- -INF   # M*fi
+		}else{# diag function fails when it is one row
+			LHS[(nr+2*nIrrev+2)   :(nr+2*nIrrev+nRev+1)  ,((nc+nIrrev+1):(nc+nIrrev+nRev))   ] <- -INF
+		}
+		#---------------------------------------------------------#
+		#  Constraint 2: Tf*fi-vi-My<=0
+        ii=matrix(c((nr+2*nIrrev+nRev+2)   :(nr+2*nIrrev+2*nRev+1)  ,((1:nc)[is_rev] )),ncol=2)
+        LHS[ii]<- -1
+		if(nRev>1){
+			diag(LHS[(nr+2*nIrrev+nRev+2)   :(nr+2*nIrrev+2*nRev+1)  ,((nc+nIrrev+1):(nc+nIrrev+nRev))   ]) <- Tf   # Tf*fi
+			diag(LHS[(nr+2*nIrrev+nRev+2)   :(nr+2*nIrrev+2*nRev+1)  ,((nc+nIrrev+nRev+1):(nc+nIrrev+2*nRev))   ]) <- -INF   # -M*yi
+		}else{# diag function fails when it is one row
+			LHS[(nr+2*nIrrev+nRev+2)   :(nr+2*nIrrev+2*nRev+1)  ,((nc+nIrrev+1):(nc+nIrrev+nRev))   ] <- Tf   # Tf*fi
+			LHS[(nr+2*nIrrev+nRev+2)   :(nr+2*nIrrev+2*nRev+1)  ,((nc+nIrrev+nRev+1):(nc+nIrrev+2*nRev))   ] <- -INF   # -M*yi
 		}
-	}
-       # ---------------------------------------------
-            # lower and upper bounds for COLUMNS(variables) and ROWS(constraints)
        # ---------------------------------------------
+    #  Constraint 3: -vi-M*fi<=Tf
+       ii=matrix(c((nr+2*nIrrev+2*nRev+2)   :(nr+2*nIrrev+3*nRev+1)  ,((1:nc)[is_rev] )),ncol=2)
+       LHS[ii]<- -1
+	    
+		if(nRev>1){
+			diag(LHS[(nr+2*nIrrev+2*nRev+2)   :(nr+2*nIrrev+3*nRev+1)  ,((nc+nIrrev+1):(nc+nIrrev+nRev))   ]) <- -INF   # M*fi
+		}else{# diag function fails when it is one row
+			LHS[(nr+2*nIrrev+2*nRev+2)   :(nr+2*nIrrev+3*nRev+1)  ,((nc+nIrrev+1):(nc+nIrrev+nRev))   ] <- -INF
+		}
+		#---------------------------------------------------------#
+		#  Constraint 4: Tf*fi+vi-M(1-y)<=0
+        ii=matrix(c((nr+2*nIrrev+3*nRev+2)   :(nr+2*nIrrev+4*nRev+1)  ,((1:nc)[is_rev] )),ncol=2)
+        LHS[ii]<- 1
+		if(nRev>1){
+			diag(LHS[(nr+2*nIrrev+3*nRev+2)   :(nr+2*nIrrev+4*nRev+1)  ,((nc+nIrrev+1):(nc+nIrrev+nRev))   ]) <- Tf   # Tf*fi
+			diag(LHS[(nr+2*nIrrev+3*nRev+2)   :(nr+2*nIrrev+4*nRev+1)  ,((nc+nIrrev+nRev+1):(nc+nIrrev+2*nRev))   ]) <- INF   # M*yi
+		}else{# diag function fails when it is one row
+			LHS[(nr+2*nIrrev+3*nRev+2)   :(nr+2*nIrrev+4*nRev+1)  ,((nc+nIrrev+1):(nc+nIrrev+nRev))   ] <- Tf   # Tf*fi
+			LHS[(nr+2*nIrrev+3*nRev+2)   :(nr+2*nIrrev+4*nRev+1)  ,((nc+nIrrev+nRev+1):(nc+nIrrev+2*nRev))   ] <- INF   # M*yi
+		}
+	}#nRev
+	#---------------------------------------------------------#
+	         # lower and upper bounds for COLUMNS(variables) and ROWS(constraints)
+    # ---------------------------------------------
      
-     	lower  <- c(lowbnd(model), rep(0, gpr_rxn), rep(0, ng))
-     	upper  <- c(uppbnd(model), rep(1, gpr_rxn), rep(1, ng))
+     	lower  <- c(lowbnd(model), rep(0, nIrrev+2*nRev), rep(0, nvg))
+     	upper  <- c(uppbnd(model), rep(1, nIrrev+2*nRev), rep(1, nvg))
      	#RHS 1:nr+1 : as FBA model , FB,      ?? why -2*ub!!
 		#rhs(model) : removed from model, 30/3/2013
-     	#rlower <- c(rhs(model), FB, -2*uppbnd(model)[gpr_rxn_ind],-2*uppbnd(model)[gpr_rxn_ind],rep(0, gpr_rxn))  # ,rep(0,ngpr)
-    	#rupper <- c(rhs(model), FB, rep(0, 3*gpr_rxn))  #,rep(0,ngpr)
-        rlower <- c(rep(0,nr), FB, -2*uppbnd(model)[gpr_rxn_ind],-2*uppbnd(model)[gpr_rxn_ind],rep(0, gpr_rxn))  # ,rep(0,ngpr)
-    	rupper <- c(rep(0,nr), FB, rep(0, 3*gpr_rxn))  #,rep(0,ngpr)
+     	#rlower <- c(rhs(model), FB, -2*uppbnd(model)[gpr_rxn_ind],-2*uppbnd(model)[gpr_rxn_ind],rep(0, nGpr))  # ,rep(0,ngpr)
+    	#rupper <- c(rhs(model), FB, rep(0, 3*nGpr))  #,rep(0,ngpr)
+        rlower <- c(rep(0,nr), FB, rep(-INF,nIrrev), rep(-INF,nIrrev), rep(-2*INF,nRev), rep(-2*INF,nRev), rep(-2*INF,nRev), rep(-INF,nRev),rep(0,nGpr))  # ,rep(0,ngpr)
+    	rupper <- c(rep(0,nr), FB, rep(Tf,nIrrev),   rep(0,nIrrev),    rep(Tf,nRev),     rep(0,nRev),      rep(Tf,nRev),     rep(INF,nRev),rep(0,nGpr))  #,rep(0,ngpr)
         
-		gprtype=rep("E",gpr_rxn);
-     # constraints of gpr: NOT is not considered
+		gprtype=rep("E",nGpr);
+    #-----------------------------------------------------------------------------------------#
+	# constraints of gpr: NOT is not considered
 	message("start gpr....");
 	 lastRow=dim(LHS)[1];
 
-	 row_i=nr+2*gpr_rxn+1;
-
-	for(i in 1:gpr_rxn){  # i : is the rxn number
-		 rl=rules[i];
+	 row_i=nr+2*nIrrev+4*nRev+1;
+     rxn_map=c(which(is_irrev),which(is_rev))# mapping from rxn to fi:at first irreversible rxns then reversible
+	for(i in 1:nGpr){  # i : is the rxn number
+		 #rl=rules[i];
+		 rl=gpr(model)[rxn_map[i]];
 # 				# search for brackets : add aux variable for each bracket  in the same way as above
 				# Consider only SUM of PRODUCT (AND to [OR]), without NOT, one level
 		row_i=row_i+1;
-		print(c(i,rl))# may make a problem if gene name contains 'or' like YOR123
+		if (verboseMode > 2) {
+			print(c(i,rxn_map[i],rl))
+		}
+		# may make a problem if gene name contains 'or' like YOR123
 		#replace )or with ) or & or( with or (
 		rl=gsub("\\)"," ) ",rl)# 
 		rl=gsub("\\("," ( ",rl)# 
 
 		pr=lapply(strsplit(unlist(strsplit(rl," or "))," and "),function(x) gsub("[() ]","",x))
 		if( length(pr)==1) {# no OR (only one term) 
-			LHS[row_i,nc+gpr_rxn+which(geneNames %in% pr[[1]])]=1;
+			#LHS[row_i,nc+nGpr+nRev+which(geneNames %in% pr[[1]])]=1;
+			    for(gene in pr[[1]]){
+					#get the corresponding col of the gene
+					generow=geneCol[geneCol[,"gene"]==gene & is.na(geneCol[,"rxn"]),]
+					if(generow[4]==1){#if cnt==1
+						colind=as.numeric(generow[3])
+					}else{
+						colind=as.numeric(geneCol[geneCol[,"gene"]==gene & geneCol[,"rxn"]==react_id(model)[rxn_map[i]] & !is.na(geneCol[,"rxn"]),"Col"])
+					}
+					LHS[row_i,colind]=1;
+				}
 			LHS[row_i,nc+i]=-length(pr[[1]]);  # this is for the rxn
 			#rlower[row_i]=-0.1; 
 			rupper[row_i]=length(pr[[1]] )-1;#+0.1
@@ -234,7 +318,19 @@ return(eqn)
 		     gprtype[i]="R";
 		     for( p in 1:length(pr)){
                      	 if( length(pr[[p]])==1 ){ 
-			              LHS[row_i,nc+gpr_rxn+which( geneNames %in% pr[[p]] ) ]=-1;
+			                #LHS[row_i,nc+nGpr+nRev+which( geneNames %in% pr[[p]] ) ]=-1;
+							gene=pr[[p]]
+							#get the corresponding col of the gene							
+							generow=geneCol[geneCol[,"gene"]==gene & is.na(geneCol[,"rxn"]),]
+							#print(generow)
+							#print(react_id(model)[rxn_map[i]])
+							if(generow[4]==1){#if cnt==1
+								colind=as.numeric(generow[3])
+							}else{
+								colind=as.numeric(geneCol[geneCol[,"gene"]==gene & geneCol[,"rxn"]==react_id(model)[rxn_map[i]] & !is.na(geneCol[,"rxn"]),"Col"])
+							}	
+							#print(colind)
+							LHS[row_i,colind]=-1;					
 			             }else{       # add auxiliary variable : add row for it then column update new row and SUM row
 							  crow=Matrix::Matrix(0, nrow = 1, ncol =dim(LHS)[2])
 							  LHS=rBind(LHS,crow)
@@ -243,15 +339,43 @@ return(eqn)
 							  ccol=Matrix::Matrix(0, nrow = lastRow, ncol =1)
 							  LHS=cBind(LHS,ccol) # new column
 							  lower=c(lower,0);	  upper=c(upper,1);
-							  LHS[lastRow,dim(LHS)[2]]=-length(pr[[p]]);  
-							  LHS[lastRow,nc+gpr_rxn+which(geneNames %in% pr[[p]])]=1;
-							  rupper[lastRow]=length(pr[[p]])-1;
-							  LHS[row_i,dim(LHS)[2]]=-1;  # add the Aux variable to Sum row
+							  LHS[lastRow,dim(LHS)[2]]=-length(pr[[p]]);  #Coef of Aux
+							  #LHS[lastRow,nc+nGpr+nRev+which(geneNames %in% pr[[p]])]=1;
+								for(gene in pr[[p]]){
+									#get the corresponding col of the gene
+									generow=geneCol[geneCol[,"gene"]==gene & is.na(geneCol[,"rxn"]),]
+									if(generow[4]==1){#if cnt==1
+										colind=as.numeric(generow[3])
+									}else{
+										colind=as.numeric(geneCol[geneCol[,"gene"]==gene & geneCol[,"rxn"]==react_id(model)[rxn_map[i]] & !is.na(geneCol[,"rxn"]),"Col"])
+									}
+									LHS[lastRow,colind]=1;
+								}
+							  rupper[lastRow]=length(pr[[p]])-1;#RHS of Aux constraint
+							  LHS[row_i,dim(LHS)[2]]=-1;  # add the Aux variable to the Sum row
 							}
 			}#for p
 		}#if len
        }# for i
-
+######################
+# Constraints of multifunctional enzymes
+######################
+  aux_cnt=dim(LHS)[1]-(nr+2*nIrrev+4*nRev+1);
+  mrg=geneCol[is.na(geneCol[,"rxn"]) & geneCol[,"cnt"]>1 & !is.na(geneCol[,"inputStat"]),];
+  if(length(mrg[,1])>0){
+	  for(gr in (1:length(mrg[,1]))){
+	  #Add constraint x=x1 or x2 or ... xn, 0<= n*x-x1-x2...<=n-1
+		  crow=Matrix::Matrix(0, nrow = 1, ncol =dim(LHS)[2])
+		  LHS=rBind(LHS,crow)
+		  rlower=c(rlower,0); 				  rupper=c(rupper,as.numeric(mrg[gr,"cnt"])-1);
+		  lastRow=dim(LHS)[1]#lastRow+1;
+		  LHS[lastRow,as.numeric(mrg[gr,"Col"])]=as.numeric(mrg[gr,"cnt"])
+		  sc=as.numeric(geneCol[!is.na(geneCol[,"rxn"]) & geneCol[,"gene"]==mrg[gr,"gene"],"Col"]);
+		  LHS[lastRow,sc]=-1
+	  }
+   }
+	mr_cnt=dim(LHS)[1]-(nr+2*nIrrev+4*nRev+1+aux_cnt)
+###############################
        message("gpr ... ... OK")
        # ---------------------------------------------
        # objective function
@@ -264,15 +388,20 @@ return(eqn)
         
     num_constr=dim(LHS)[1];
 	num_var=dim(LHS)[2];
-	aux_cnt=num_constr-(nr+3*gpr_rxn+1);
-	#print(paste("no cons:",num_constr,"no var:",num_var));
-    cobj <- c(rep(0, nc+gpr_rxn),ifelse(is.na(x),0,ifelse(x,-1,1)) ,rep(0, num_var-(nc+gpr_rxn+ng))) # NA->0 
 	
-	cNames=paste(c(rep("x",nc),rep("rxn",gpr_rxn),rep("g",ng),rep("Aux",(num_var-(nc+gpr_rxn+ng)))),
-		 		               c(1:nc,which(gpr_rxn_ind),1:ng,1:(num_var-(nc+gpr_rxn+ng))),sep="" ) ;
-        # ---------------------------------------------
+	#print(paste("no cons:",num_constr,"no var:",num_var));
+	#x:expression state true,false,na
+    #cobj <- c(rep(0, nc+nGpr+nRev),ifelse(is.na(x),0,ifelse(x,-1,1)) ,rep(0, num_var-(nc+nGpr+nRev+ng))) # NA->0
+	cobj <- rep(0,num_var);
+	cobj[as.numeric(geneCol[is.na(geneCol[,"rxn"]),"Col"])] <- ifelse(is.na(x),0,ifelse(x,-1,1));
+	
+	cNames=paste(c(rep("x",nc),rep("Irv",nIrrev),rep("Rev",nRev),rep("y",nRev),rep("",nvg)),
+		 		               c(1:nc,which(is_irrev),which(is_rev),which(is_rev),geneCol[,"varname"]),sep="" ) ;
+	if((num_var-(nc+nGpr+nRev+nvg))>0){cNames=c(cNames,paste(rep("Aux",(num_var-(nc+nGpr+nRev+nvg))),1:(num_var-(nc+nGpr+nRev+nvg)),sep=""))}
+	#browser();#Q to exit
+	# ---------------------------------------------
         # build problem object
-        # ---------------------------------------------
+    # ---------------------------------------------
 	switch(solver,
             # ----------------------- #
             "glpkAPI" = {
@@ -281,15 +410,15 @@ return(eqn)
                
                 out[[1]] <-  glpkAPI::addRowsGLPK(prob, nrows=num_constr)
 		outj <- glpkAPI::addColsGLPK(prob, ncols=num_var)
-		#setColNameGLPK(prob,c(1:(num_var)),paste(c(rep("x",nc),rep("rxn",gpr_rxn),rep("g",ng),rep("Aux",(num_var-(nc+gpr_rxn+ng)))),
-		#               c(1:nc,which(gpr_rxn_ind),1:ng,1:(num_var-(nc+gpr_rxn+ng))),sep="" ) );
+		#setColNameGLPK(prob,c(1:(num_var)),paste(c(rep("x",nc),rep("rxn",nGpr),rep("g",ng),rep("Aux",(num_var-(nc+nGpr+ng)))),
+		#               c(1:nc,which(gpr_rxn_ind),1:ng,1:(num_var-(nc+nGpr+ng))),sep="" ) );
 		mapply(glpkAPI::setColNameGLPK, j = c(1:(num_var)), cname = cNames, MoreArgs = list(lp = prob));
 
 		glpkAPI::setObjDirGLPK(prob, glpkAPI::GLP_MIN);
                 ## note: when FX or LO value taken from lb and when UP take from UP and when R
-               rtype <- c(rep(glpkAPI::GLP_FX, nr+1), rep(glpkAPI::GLP_UP, 2*gpr_rxn), ifelse(gprtype=="E",glpkAPI::GLP_FX,glpkAPI::GLP_DB),
-			   rep(glpkAPI::GLP_FX,aux_cnt))
-	       
+               rtype <- c(rep(glpkAPI::GLP_FX, nr),glpkAPI::GLP_LO, rep(glpkAPI::GLP_UP, 2*nIrrev+4*nRev), ifelse(gprtype=="E",glpkAPI::GLP_FX,glpkAPI::GLP_DB),
+			   rep(glpkAPI::GLP_DB,aux_cnt),rep(glpkAPI::GLP_DB,mr_cnt))
+	       print(table(rtype));
 	        # set the right hand side Sv = b
 
 	       out[[4]] <- glpkAPI::setRowsBndsGLPK(prob, c(1:num_constr), lb=rlower, ub=rupper,type=rtype )
@@ -323,11 +452,12 @@ return(eqn)
 		 	print(glpkAPI::status_codeGLPK(lp_stat));
 		 	}
 		lp_obj=glpkAPI::mipObjValGLPK(prob);
-		colst=glpkAPI::mipColsValGLPK(prob);
-		newFlux=colst
+		colVal=glpkAPI::mipColsValGLPK(prob);
+		newFlux=colVal
+		colst=cbind(cNames,val=colVal,lower,upper,ctype);
 	       ## --- 
-	        newFlux=floor(newFlux/Tf)*Tf#sybil:::.floorValues(newFlux,tol=Tf);
-	        sol_geneStat=ifelse(newFlux[(nc+gpr_rxn+1) : (nc+gpr_rxn+ng)]==1,"ON","OFF");
+	        #newFlux=floor(newFlux/Tf)*Tf#sybil:::.floorValues(newFlux,tol=Tf);
+	        sol_geneStat=ifelse(newFlux[(nc+nGpr+nRev+1) : (nc+nGpr+nRev+nvg)]==1,"ON","OFF");
 	        newFlux=newFlux[1:nc];
 	        newStat=ifelse(abs(newFlux)>Tf,1,0);
 
@@ -338,8 +468,8 @@ return(eqn)
 				 prob <- cplexAPI::openProbCPLEX()
 				 out <- cplexAPI::setIntParmCPLEX(prob$env, cplexAPI::CPX_PARAM_SCRIND, cplexAPI::CPX_OFF)		                
                 
-                # when R: rngval+rhs   to rhs   (rngval<0)
-                rtype <- c(rep("E",nr+1),  rep("L", 2*gpr_rxn),gprtype,rep("R",aux_cnt))
+                # when R: rngval+rhs   to rhs   (rngval<0), E,L,R,U?
+                rtype <- c(rep("E",nr),"G",  rep("L", 2*nIrrev+4*nRev),gprtype,rep("R",aux_cnt),rep("R",mr_cnt))
                 prob$lp<- cplexAPI::initProbCPLEX(prob$env)
 				cplexAPI::chgProbNameCPLEX(prob$env, prob$lp, "eFBA gene cplex");
                  out[[1]] <- cplexAPI::newRowsCPLEX(prob$env, prob$lp,
@@ -354,13 +484,7 @@ return(eqn)
                                    ia  = TMPmat@i,
                                    ja  = TMPmat@j,
                                    ra  = TMPmat@x);				
-		  #    nzLHS=nzijr(LHS);        
-                # out[[3]] <- chgCoefListCPLEX(prob$env, prob$lp,
-                                             # nzLHS$ne,
-                                             # nzLHS$ia  - 1,
-                                            # nzLHS$ja - 1,
-                                             # nzLHS$ar)
-          
+	      
                 if(num_var>nc){ ctype<-c(rep('C', nc), rep('B',num_var - nc));# integer variables
 				}else{ctype<-rep('C', nc);}
 				status = cplexAPI::copyColTypeCPLEX (prob$env, prob$lp, ctype);
@@ -386,12 +510,12 @@ return(eqn)
               lp_obj=sol$objval;
                lp_stat   <- cplexAPI::getStatCPLEX(prob$env, prob$lp)
               
-               colst=sol$x;
+               colst=cbind(cNames,val=sol$x,lower,upper,ctype);
                  
                #have flux and gene ON /flux and gene OFF?
                newFlux=floor(sol$x/Tf)*Tf#sybil:::.floorValues(sol$x,tol=Tf);
                newFlux=newFlux[1:nc];
-               sol_geneStat=ifelse(sol$x[(nc+gpr_rxn+1):(nc+gpr_rxn+ng)]==1,"ON","OFF");
+               sol_geneStat=ifelse(sol$x[(nc+nGpr+nRev+1):(nc+nGpr+nRev+nvg)]==1,"ON","OFF");
                newStat=ifelse(abs(newFlux)>Tf,1,0);
                # when using binary variables as indicators: .floorValues(.ceilValues(newFlux[(nc+1):(2*nc)],tol=Tf/10),tol=Tf);
            },
@@ -401,15 +525,15 @@ return(eqn)
             }
         )
         
- 	       print(sprintf("Number of Rxns with gene ON=%d ",length(rxnStatus[rxnStatus==-1]) ));
+ 	       print(sprintf("Number of Rxns with gene rule ON=%d ",length(rxnStatus[rxnStatus==-1]) ));
 
 	       print(sprintf("Rxns with gene OFF=%d",length(rxnStatus[rxnStatus==1]) ));  
 	       print(sprintf("Total Nr of selected rules=%d ",length(rxnStatus[rxnStatus==0]) ));
 	       print(sprintf("Total number of rxns: %d",length(rxnStatus) ));
-	       print(sprintf("The differnece is: %.0f ", (lp_obj+length(rxnStatus[rxnStatus==-1])) ));
+	       print(sprintf("The difference from objective: %.0f ", (lp_obj+length(rxnStatus[rxnStatus==-1])) ));#may contain a problem
 
- 		origFlux=floor((fluxes(orig_sol)[fldind(orig_sol)])*Tf)/Tf;#sybil:::.floorValues(fluxes(orig_sol)[fldind(orig_sol)],tol=Tf);
- 		
+ 		origFlux=getFluxDist(orig_sol);
+		
  		#excReact = findExchReact(model)[1];# 1 is position
 		#excReactPos=react_pos(excReact$exchange);
         excReact = findExchReact(model);
@@ -418,19 +542,16 @@ return(eqn)
         is_excReact=((c(1:length(react_id(model)))) %in% excReactPos);
                
                 origStat=ifelse(abs(origFlux)>Tf,1,0);
+				print(sprintf("Difference from FBA fluxes: %d",sum(abs(origStat-newStat))));
+				
                rxnstname=ifelse(rxnStatus==0,"No rule/Unk",ifelse(rxnStatus==-1,"ON","OFF"));
                rxn_st <- cbind(gpr=gpr(model),react=react_id(model),reactName=react_name(model),is_excReact=is_excReact,
                                  expr=rxnstname,lb=lowbnd(model),
                                        #  eqns=sapply(c(1:length(react_id(model))),function(x)getRxnEqn(x)) ,
                                           origFlux,origStat, newFlux,newStat,difference=abs(origStat-newStat));
  
-               gene_st <- cbind(geneLocus=allGenes(model),inputStat=x,SolStat=sol_geneStat)
-#               write.csv(rxn_st,"rst.csv");
-#              write.csv(gene_st,"gene.csv");
-               
-#               output_rep=table(list(expr=rxnstname,orig=origStat,newsol=newStat));
-#               write.csv(output_rep,"eFBA_report.csv");
-
+               #gene_st <- cbind(geneLocus=allGenes(model),inputStat=x,SolStat=sol_geneStat)
+			   gene_st <- cbind(varname=geneCol[,"varname"],inputStat=geneCol[,"inputStat"],SolStat=sol_geneStat)
  
   remove(prob);
 #----------------------------------------  ***  --------------------------------#
@@ -446,22 +567,8 @@ return(eqn)
 			  )
 	return(optsol);
 }
-
-
-
-#if(solver=="cplex"){
-#   writeProbCPLEX(prob$env, prob$lp,"e:\\sybil\\Ec_eFBA_f1.lp");
-#   }
-#   else{
-#   writeLPGLPK(prob,"E:\\Sybil\\Ec_FBAGLPK.lp");
-#   }
-#----------------------------------------  ***  --------------------------------#
-#check <- loadProblemDataEFBA(lp, model, FB, expressionData=expressionData, nCols=react_num(model)*2, 
-#                                                 nRows= met_num(model)+1+react_num(model)*2);
-# Test gpr
-# rl="(yor123 and abcd)or yfl012 or (abc and (xyz or wxc))"
-# rl=gsub("\\)"," ) ",rl)# 
-# rl=gsub("\\("," ( ",rl)# 
-# rl
-# pr=lapply(strsplit(unlist(strsplit(rl," or "))," and "),function(x) gsub("[() ]","",x))
-# pr
+#               write.csv(rxn_st,"rst.csv");
+#              write.csv(gene_st,"gene.csv");
+               
+#               output_rep=table(list(expr=rxnstname,orig=origStat,newsol=newStat));
+#               write.csv(output_rep,"eFBA_report.csv");
